@@ -21,8 +21,10 @@
 
 import collections
 import sys
+import warnings
+
 from gi.repository import GObject
-from ..overrides import override, strip_boolean_result
+from ..overrides import override, strip_boolean_result, deprecated_init
 from ..module import get_introspection_module
 from gi import PyGIDeprecationWarning
 
@@ -38,7 +40,6 @@ Gtk = get_introspection_module('Gtk')
 __all__ = []
 
 if Gtk._version == '2.0':
-    import warnings
     warn_msg = "You have imported the Gtk 2.0 module.  Because Gtk 2.0 \
 was not designed for use with introspection some of the \
 interfaces and API will fail.  As such this is not supported \
@@ -49,6 +50,12 @@ python module to use with Gtk 2.0"
     warnings.warn(warn_msg, RuntimeWarning)
 
 
+class PyGTKDeprecationWarning(PyGIDeprecationWarning):
+    pass
+
+__all__.append('PyGTKDeprecationWarning')
+
+
 def _construct_target_list(targets):
     """Create a list of TargetEntry items from a list of tuples in the form (target, flags, info)
 
@@ -73,12 +80,12 @@ class Widget(Gtk.Widget):
         return super(Widget, self).render_icon(stock_id, size, detail)
 
     def drag_dest_set_target_list(self, target_list):
-        if not isinstance(target_list, Gtk.TargetList):
+        if (target_list is not None) and (not isinstance(target_list, Gtk.TargetList)):
             target_list = Gtk.TargetList.new(_construct_target_list(target_list))
         super(Widget, self).drag_dest_set_target_list(target_list)
 
     def drag_source_set_target_list(self, target_list):
-        if not isinstance(target_list, Gtk.TargetList):
+        if (target_list is not None) and (not isinstance(target_list, Gtk.TargetList)):
             target_list = Gtk.TargetList.new(_construct_target_list(target_list))
         super(Widget, self).drag_source_set_target_list(target_list)
 
@@ -124,24 +131,27 @@ __all__.append("Editable")
 
 
 class Action(Gtk.Action):
-    def __init__(self, name, label, tooltip, stock_id, **kwds):
-        Gtk.Action.__init__(self, name=name, label=label, tooltip=tooltip, stock_id=stock_id, **kwds)
+    __init__ = deprecated_init(Gtk.Action.__init__,
+                               arg_names=('name', 'label', 'tooltip', 'stock_id'),
+                               category=PyGTKDeprecationWarning)
 
 Action = override(Action)
 __all__.append("Action")
 
 
 class RadioAction(Gtk.RadioAction):
-    def __init__(self, name, label, tooltip, stock_id, value, **kwds):
-        Gtk.RadioAction.__init__(self, name=name, label=label, tooltip=tooltip, stock_id=stock_id, value=value, **kwds)
+    __init__ = deprecated_init(Gtk.RadioAction.__init__,
+                               arg_names=('name', 'label', 'tooltip', 'stock_id', 'value'),
+                               category=PyGTKDeprecationWarning)
 
 RadioAction = override(RadioAction)
 __all__.append("RadioAction")
 
 
 class ActionGroup(Gtk.ActionGroup):
-    def __init__(self, name, **kwds):
-        super(ActionGroup, self).__init__(name=name, **kwds)
+    __init__ = deprecated_init(Gtk.ActionGroup.__init__,
+                               arg_names=('name',),
+                               category=PyGTKDeprecationWarning)
 
     def add_actions(self, entries, user_data=None):
         """
@@ -175,7 +185,7 @@ class ActionGroup(Gtk.ActionGroup):
             raise TypeError('entries must be iterable')
 
         def _process_action(name, stock_id=None, label=None, accelerator=None, tooltip=None, callback=None):
-            action = Action(name, label, tooltip, stock_id)
+            action = Action(name=name, label=label, tooltip=tooltip, stock_id=stock_id)
             if callback is not None:
                 if user_data is None:
                     action.connect('activate', callback)
@@ -223,7 +233,7 @@ class ActionGroup(Gtk.ActionGroup):
             raise TypeError('entries must be iterable')
 
         def _process_action(name, stock_id=None, label=None, accelerator=None, tooltip=None, callback=None, is_active=False):
-            action = Gtk.ToggleAction(name, label, tooltip, stock_id)
+            action = Gtk.ToggleAction(name=name, label=label, tooltip=tooltip, stock_id=stock_id)
             action.set_active(is_active)
             if callback is not None:
                 if user_data is None:
@@ -273,7 +283,7 @@ class ActionGroup(Gtk.ActionGroup):
         first_action = None
 
         def _process_action(group_source, name, stock_id=None, label=None, accelerator=None, tooltip=None, entry_value=0):
-            action = RadioAction(name, label, tooltip, stock_id, entry_value)
+            action = RadioAction(name=name, label=label, tooltip=tooltip, stock_id=stock_id, value=entry_value)
 
             # FIXME: join_group is a patch to Gtk+ 3.0
             #        otherwise we can't effectively add radio actions to a
@@ -329,29 +339,28 @@ __all__.append('ComboBox')
 
 
 class Box(Gtk.Box):
-    def __init__(self, homogeneous=False, spacing=0, **kwds):
-        super(Box, self).__init__(**kwds)
-        self.set_homogeneous(homogeneous)
-        self.set_spacing(spacing)
+    __init__ = deprecated_init(Gtk.Box.__init__,
+                               arg_names=('homogeneous', 'spacing'),
+                               category=PyGTKDeprecationWarning)
 
 Box = override(Box)
 __all__.append('Box')
 
 
 class SizeGroup(Gtk.SizeGroup):
-    def __init__(self, mode=Gtk.SizeGroupMode.VERTICAL):
-        super(SizeGroup, self).__init__(mode=mode)
+    __init__ = deprecated_init(Gtk.SizeGroup.__init__,
+                               arg_names=('mode',),
+                               deprecated_defaults={'mode': Gtk.SizeGroupMode.VERTICAL},
+                               category=PyGTKDeprecationWarning)
 
 SizeGroup = override(SizeGroup)
 __all__.append('SizeGroup')
 
 
 class MenuItem(Gtk.MenuItem):
-    def __init__(self, label=None, **kwds):
-        if label:
-            super(MenuItem, self).__init__(label=label, **kwds)
-        else:
-            super(MenuItem, self).__init__(**kwds)
+    __init__ = deprecated_init(Gtk.MenuItem.__init__,
+                               arg_names=('label',),
+                               category=PyGTKDeprecationWarning)
 
 MenuItem = override(MenuItem)
 __all__.append('MenuItem')
@@ -432,56 +441,66 @@ __all__.append('Builder')
 
 
 class Window(Gtk.Window):
-    def __init__(self, type=Gtk.WindowType.TOPLEVEL, **kwds):
-        if not initialized:
-            raise RuntimeError("Gtk couldn't be initialized")
-
-        # type is a construct-only property; if it is already set (e. g. by
-        # GtkBuilder), do not try to set it again and just ignore it
-        try:
-            self.get_property('type')
-            Gtk.Window.__init__(self, **kwds)
-        except TypeError:
-            Gtk.Window.__init__(self, type=type, **kwds)
+    __init__ = deprecated_init(Gtk.Window.__init__,
+                               arg_names=('type',),
+                               category=PyGTKDeprecationWarning)
 
 Window = override(Window)
 __all__.append('Window')
 
 
 class Dialog(Gtk.Dialog, Container):
+    _old_arg_names = ('title', 'parent', 'flags', 'buttons', '_buttons_property')
+    _init = deprecated_init(Gtk.Dialog.__init__,
+                            arg_names=('title', 'transient_for', 'flags',
+                                       'add_buttons', 'buttons'),
+                            ignore=('flags', 'add_buttons'),
+                            deprecated_aliases={'transient_for': 'parent',
+                                                'buttons': '_buttons_property'},
+                            category=PyGTKDeprecationWarning)
 
-    def __init__(self,
-                 title=None,
-                 parent=None,
-                 flags=0,
-                 buttons=None,
-                 _buttons_property=None,
-                 **kwds):
+    def __init__(self, *args, **kwargs):
 
-        # buttons is overloaded by PyGtk so we have to do the same here
-        # this breaks some subclasses of Dialog so add a _buttons_property
-        # keyword to work around this
-        if _buttons_property is not None:
-            kwds['buttons'] = _buttons_property
+        new_kwargs = kwargs.copy()
+        old_kwargs = dict(zip(self._old_arg_names, args))
+        old_kwargs.update(kwargs)
 
-        Gtk.Dialog.__init__(self, **kwds)
-        if title:
-            self.set_title(title)
-        if parent:
-            self.set_transient_for(parent)
-        if flags & Gtk.DialogFlags.MODAL:
-            self.set_modal(True)
-        if flags & Gtk.DialogFlags.DESTROY_WITH_PARENT:
-            self.set_destroy_with_parent(True)
+        # Increment the warning stacklevel for sub-classes which implement their own __init__.
+        stacklevel = 2
+        if self.__class__ != Dialog and self.__class__.__init__ != Dialog.__init__:
+            stacklevel += 1
 
-        # NO_SEPARATOR has been removed from Gtk 3
-        if hasattr(Gtk.DialogFlags, "NO_SEPARATOR") and (flags & Gtk.DialogFlags.NO_SEPARATOR):
-            self.set_has_separator(False)
-            import warnings
-            warnings.warn("Gtk.DialogFlags.NO_SEPARATOR has been depricated since Gtk+-3.0", PyGIDeprecationWarning)
+        # buttons was overloaded by PyGtk but is needed for Gtk.MessageDialog
+        # as a pass through, so type check the argument and give a deprecation
+        # when it is not of type Gtk.ButtonsType
+        add_buttons = old_kwargs.get('buttons', None)
+        if add_buttons is not None and not isinstance(add_buttons, Gtk.ButtonsType):
+            warnings.warn('The "buttons" argument must be a Gtk.ButtonsType enum value. '
+                          'Please use the "add_buttons" method for adding buttons. '
+                          'See: https://wiki.gnome.org/PyGObject/InitializerDeprecations',
+                          PyGTKDeprecationWarning, stacklevel=stacklevel)
+            if 'buttons' in new_kwargs:
+                del new_kwargs['buttons']
+        else:
+            add_buttons = None
 
-        if buttons is not None:
-            self.add_buttons(*buttons)
+        flags = old_kwargs.get('flags', 0)
+        if flags:
+            warnings.warn('The "flags" argument for dialog construction is deprecated. '
+                          'Please use initializer keywords: modal=True and/or destroy_with_parent=True. '
+                          'See: https://wiki.gnome.org/PyGObject/InitializerDeprecations',
+                          PyGTKDeprecationWarning, stacklevel=stacklevel)
+
+            if flags & Gtk.DialogFlags.MODAL:
+                new_kwargs['modal'] = True
+
+            if flags & Gtk.DialogFlags.DESTROY_WITH_PARENT:
+                new_kwargs['destroy_with_parent'] = True
+
+        self._init(*args, **new_kwargs)
+
+        if add_buttons:
+            self.add_buttons(*add_buttons)
 
     action_area = property(lambda dialog: dialog.get_action_area())
     vbox = property(lambda dialog: dialog.get_content_area())
@@ -515,29 +534,12 @@ __all__.append('Dialog')
 
 
 class MessageDialog(Gtk.MessageDialog, Dialog):
-    def __init__(self,
-                 parent=None,
-                 flags=0,
-                 message_type=Gtk.MessageType.INFO,
-                 buttons=Gtk.ButtonsType.NONE,
-                 message_format=None,
-                 **kwds):
-
-        if message_format:
-            kwds['text'] = message_format
-
-        # type keyword is used for backwards compat with PyGTK
-        if 'type' in kwds:
-            import warnings
-            warnings.warn("The use of the keyword type as a parameter of the Gtk.MessageDialog constructor has been depricated. Please use message_type instead.", PyGIDeprecationWarning)
-            message_type = kwds.pop('type')
-
-        Gtk.MessageDialog.__init__(self,
-                                   _buttons_property=buttons,
-                                   message_type=message_type,
-                                   parent=parent,
-                                   flags=flags,
-                                   **kwds)
+    __init__ = deprecated_init(Gtk.MessageDialog.__init__,
+                               arg_names=('parent', 'flags', 'message_type',
+                                          'buttons', 'message_format'),
+                               deprecated_aliases={'text': 'message_format',
+                                                   'message_type': 'type'},
+                               category=PyGTKDeprecationWarning)
 
     def format_secondary_text(self, message_format):
         self.set_property('secondary-use-markup', False)
@@ -551,70 +553,49 @@ MessageDialog = override(MessageDialog)
 __all__.append('MessageDialog')
 
 
-class AboutDialog(Gtk.AboutDialog):
-    def __init__(self, **kwds):
-        Gtk.AboutDialog.__init__(self, **kwds)
-
-AboutDialog = override(AboutDialog)
-__all__.append('AboutDialog')
-
-
 class ColorSelectionDialog(Gtk.ColorSelectionDialog):
-    def __init__(self, title=None, **kwds):
-        Gtk.ColorSelectionDialog.__init__(self, title=title, **kwds)
+    __init__ = deprecated_init(Gtk.ColorSelectionDialog.__init__,
+                               arg_names=('title',),
+                               category=PyGTKDeprecationWarning)
 
 ColorSelectionDialog = override(ColorSelectionDialog)
 __all__.append('ColorSelectionDialog')
 
 
 class FileChooserDialog(Gtk.FileChooserDialog):
-    def __init__(self,
-                 title=None,
-                 parent=None,
-                 action=Gtk.FileChooserAction.OPEN,
-                 buttons=None,
-                 **kwds):
-        Gtk.FileChooserDialog.__init__(self,
-                                       action=action,
-                                       title=title,
-                                       parent=parent,
-                                       buttons=buttons,
-                                       **kwds)
+    __init__ = deprecated_init(Gtk.FileChooserDialog.__init__,
+                               arg_names=('title', 'parent', 'action', 'buttons'),
+                               category=PyGTKDeprecationWarning)
+
 FileChooserDialog = override(FileChooserDialog)
 __all__.append('FileChooserDialog')
 
 
 class FontSelectionDialog(Gtk.FontSelectionDialog):
-    def __init__(self, title=None, **kwds):
-        Gtk.FontSelectionDialog.__init__(self, title=title, **kwds)
+    __init__ = deprecated_init(Gtk.FontSelectionDialog.__init__,
+                               arg_names=('title',),
+                               category=PyGTKDeprecationWarning)
 
 FontSelectionDialog = override(FontSelectionDialog)
 __all__.append('FontSelectionDialog')
 
 
 class RecentChooserDialog(Gtk.RecentChooserDialog):
-    def __init__(self,
-                 title=None,
-                 parent=None,
-                 manager=None,
-                 buttons=None,
-                 **kwds):
-
-        Gtk.RecentChooserDialog.__init__(self,
-                                         recent_manager=manager,
-                                         title=title,
-                                         parent=parent,
-                                         buttons=buttons,
-                                         **kwds)
+    # Note, the "manager" keyword must work across the entire 3.x series because
+    # "recent_manager" is not backwards compatible with PyGObject versions prior to 3.10.
+    __init__ = deprecated_init(Gtk.RecentChooserDialog.__init__,
+                               arg_names=('title', 'parent', 'recent_manager', 'buttons'),
+                               deprecated_aliases={'recent_manager': 'manager'},
+                               category=PyGTKDeprecationWarning)
 
 RecentChooserDialog = override(RecentChooserDialog)
 __all__.append('RecentChooserDialog')
 
 
 class IconView(Gtk.IconView):
-
-    def __init__(self, model=None, **kwds):
-        Gtk.IconView.__init__(self, model=model, **kwds)
+    __init__ = deprecated_init(Gtk.IconView.__init__,
+                               arg_names=('model',),
+                               category=PyGTKDeprecationWarning)
 
     get_item_at_pos = strip_boolean_result(Gtk.IconView.get_item_at_pos)
     get_visible_range = strip_boolean_result(Gtk.IconView.get_visible_range)
@@ -625,9 +606,9 @@ __all__.append('IconView')
 
 
 class ToolButton(Gtk.ToolButton):
-
-    def __init__(self, stock_id=None, **kwds):
-        Gtk.ToolButton.__init__(self, stock_id=stock_id, **kwds)
+    __init__ = deprecated_init(Gtk.ToolButton.__init__,
+                               arg_names=('stock_id',),
+                               category=PyGTKDeprecationWarning)
 
 ToolButton = override(ToolButton)
 __all__.append('ToolButton')
@@ -919,8 +900,9 @@ __all__.append('TreeSortable')
 
 
 class TreeModelSort(Gtk.TreeModelSort):
-    def __init__(self, model, **kwds):
-        Gtk.TreeModelSort.__init__(self, model=model, **kwds)
+    __init__ = deprecated_init(Gtk.TreeModelSort.__init__,
+                               arg_names=('model',),
+                               category=PyGTKDeprecationWarning)
 
 TreeModelSort = override(TreeModelSort)
 __all__.append('TreeModelSort')
@@ -1176,7 +1158,6 @@ __all__.append('TreePath')
 
 
 class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
-
     def __init__(self, *column_types):
         Gtk.TreeStore.__init__(self)
         self.set_column_types(column_types)
@@ -1256,11 +1237,9 @@ __all__.append('TreeStore')
 
 
 class TreeView(Gtk.TreeView, Container):
-
-    def __init__(self, model=None):
-        Gtk.TreeView.__init__(self)
-        if model:
-            self.set_model(model)
+    __init__ = deprecated_init(Gtk.TreeView.__init__,
+                               arg_names=('model',),
+                               category=PyGTKDeprecationWarning)
 
     get_path_at_pos = strip_boolean_result(Gtk.TreeView.get_path_at_pos)
     get_visible_range = strip_boolean_result(Gtk.TreeView.get_visible_range)
@@ -1356,66 +1335,79 @@ __all__.append('TreeSelection')
 
 
 class Button(Gtk.Button, Container):
-    def __init__(self, label=None, stock=None, use_stock=False, use_underline=False, **kwds):
-        if stock:
-            label = stock
-            use_stock = True
-            use_underline = True
-        Gtk.Button.__init__(self, label=label, use_stock=use_stock,
-                            use_underline=use_underline, **kwds)
+    _init = deprecated_init(Gtk.Button.__init__,
+                            arg_names=('label', 'stock', 'use_stock', 'use_underline'),
+                            ignore=('stock',),
+                            category=PyGTKDeprecationWarning,
+                            stacklevel=3)
+
+    def __init__(self, *args, **kwargs):
+        # Doubly deprecated initializer, the stock keyword is non-standard.
+        # Simply give a warning that stock items are deprecated even though
+        # we want to deprecate the non-standard keyword as well here from
+        # the overrides.
+        if 'stock' in kwargs and kwargs['stock']:
+            warnings.warn('Stock items are deprecated. '
+                          'Please use: Gtk.Button.new_with_mnemonic(label)',
+                          PyGTKDeprecationWarning, stacklevel=2)
+            new_kwargs = kwargs.copy()
+            new_kwargs['label'] = new_kwargs['stock']
+            new_kwargs['use_stock'] = True
+            new_kwargs['use_underline'] = True
+            del new_kwargs['stock']
+            Gtk.Button.__init__(self, **new_kwargs)
+        else:
+            self._init(*args, **kwargs)
+
 Button = override(Button)
 __all__.append('Button')
 
 
 class LinkButton(Gtk.LinkButton):
-    def __init__(self, uri, label=None, **kwds):
-        Gtk.LinkButton.__init__(self, uri=uri, label=label, **kwds)
+    __init__ = deprecated_init(Gtk.LinkButton.__init__,
+                               arg_names=('uri', 'label'),
+                               category=PyGTKDeprecationWarning)
 
 LinkButton = override(LinkButton)
 __all__.append('LinkButton')
 
 
 class Label(Gtk.Label):
-    def __init__(self, label=None, **kwds):
-        Gtk.Label.__init__(self, label=label, **kwds)
+    __init__ = deprecated_init(Gtk.Label.__init__,
+                               arg_names=('label',),
+                               category=PyGTKDeprecationWarning)
 
 Label = override(Label)
 __all__.append('Label')
 
 
 class Adjustment(Gtk.Adjustment):
-    def __init__(self, *args, **kwds):
-        arg_names = ('value', 'lower', 'upper',
-                     'step_increment', 'page_increment', 'page_size')
-        new_args = dict(zip(arg_names, args))
-        new_args.update(kwds)
+    _init = deprecated_init(Gtk.Adjustment.__init__,
+                            arg_names=('value', 'lower', 'upper',
+                                       'step_increment', 'page_increment', 'page_size'),
+                            deprecated_aliases={'page_increment': 'page_incr',
+                                                'step_increment': 'step_incr'},
+                            category=PyGTKDeprecationWarning,
+                            stacklevel=3)
 
-        # PyGTK compatiblity
-        if 'page_incr' in new_args:
-            new_args['page_increment'] = new_args.pop('page_incr')
-        if 'step_incr' in new_args:
-            new_args['step_increment'] = new_args.pop('step_incr')
-        Gtk.Adjustment.__init__(self, **new_args)
+    def __init__(self, *args, **kwargs):
+        self._init(*args, **kwargs)
 
         # The value property is set between lower and (upper - page_size).
         # Just in case lower, upper or page_size was still 0 when value
         # was set, we set it again here.
-        if 'value' in new_args:
-            self.set_value(new_args['value'])
+        if 'value' in kwargs:
+            self.set_value(kwargs['value'])
 
 Adjustment = override(Adjustment)
 __all__.append('Adjustment')
 
 
 class Table(Gtk.Table, Container):
-    def __init__(self, rows=1, columns=1, homogeneous=False, **kwds):
-        if 'n_rows' in kwds:
-            rows = kwds.pop('n_rows')
-
-        if 'n_columns' in kwds:
-            columns = kwds.pop('n_columns')
-
-        Gtk.Table.__init__(self, n_rows=rows, n_columns=columns, homogeneous=homogeneous, **kwds)
+    __init__ = deprecated_init(Gtk.Table.__init__,
+                               arg_names=('n_rows', 'n_columns', 'homogeneous'),
+                               deprecated_aliases={'n_rows': 'rows', 'n_columns': 'columns'},
+                               category=PyGTKDeprecationWarning)
 
     def attach(self, child, left_attach, right_attach, top_attach, bottom_attach, xoptions=Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL, yoptions=Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL, xpadding=0, ypadding=0):
         Gtk.Table.attach(self, child, left_attach, right_attach, top_attach, bottom_attach, xoptions, yoptions, xpadding, ypadding)
@@ -1425,24 +1417,27 @@ __all__.append('Table')
 
 
 class ScrolledWindow(Gtk.ScrolledWindow):
-    def __init__(self, hadjustment=None, vadjustment=None, **kwds):
-        Gtk.ScrolledWindow.__init__(self, hadjustment=hadjustment, vadjustment=vadjustment, **kwds)
+    __init__ = deprecated_init(Gtk.ScrolledWindow.__init__,
+                               arg_names=('hadjustment', 'vadjustment'),
+                               category=PyGTKDeprecationWarning)
 
 ScrolledWindow = override(ScrolledWindow)
 __all__.append('ScrolledWindow')
 
 
 class HScrollbar(Gtk.HScrollbar):
-    def __init__(self, adjustment=None, **kwds):
-        Gtk.HScrollbar.__init__(self, adjustment=adjustment, **kwds)
+    __init__ = deprecated_init(Gtk.HScrollbar.__init__,
+                               arg_names=('adjustment',),
+                               category=PyGTKDeprecationWarning)
 
 HScrollbar = override(HScrollbar)
 __all__.append('HScrollbar')
 
 
 class VScrollbar(Gtk.VScrollbar):
-    def __init__(self, adjustment=None, **kwds):
-        Gtk.VScrollbar.__init__(self, adjustment=adjustment, **kwds)
+    __init__ = deprecated_init(Gtk.VScrollbar.__init__,
+                               arg_names=('adjustment',),
+                               category=PyGTKDeprecationWarning)
 
 VScrollbar = override(VScrollbar)
 __all__.append('VScrollbar')
@@ -1460,10 +1455,9 @@ __all__.append('Paned')
 
 
 class Arrow(Gtk.Arrow):
-    def __init__(self, arrow_type, shadow_type, **kwds):
-        Gtk.Arrow.__init__(self, arrow_type=arrow_type,
-                           shadow_type=shadow_type,
-                           **kwds)
+    __init__ = deprecated_init(Gtk.Arrow.__init__,
+                               arg_names=('arrow_type', 'shadow_type'),
+                               category=PyGTKDeprecationWarning)
 
 Arrow = override(Arrow)
 __all__.append('Arrow')
@@ -1472,6 +1466,9 @@ __all__.append('Arrow')
 class IconSet(Gtk.IconSet):
     def __new__(cls, pixbuf=None):
         if pixbuf is not None:
+            warnings.warn('Gtk.IconSet(pixbuf) has been deprecated. Please use: '
+                          'Gtk.IconSet.new_from_pixbuf(pixbuf)',
+                          PyGTKDeprecationWarning, stacklevel=2)
             iconset = Gtk.IconSet.new_from_pixbuf(pixbuf)
         else:
             iconset = Gtk.IconSet.__new__(cls)
@@ -1482,10 +1479,9 @@ __all__.append('IconSet')
 
 
 class Viewport(Gtk.Viewport):
-    def __init__(self, hadjustment=None, vadjustment=None, **kwds):
-        Gtk.Viewport.__init__(self, hadjustment=hadjustment,
-                              vadjustment=vadjustment,
-                              **kwds)
+    __init__ = deprecated_init(Gtk.Viewport.__init__,
+                               arg_names=('hadjustment', 'vadjustment'),
+                               category=PyGTKDeprecationWarning)
 
 Viewport = override(Viewport)
 __all__.append('Viewport')
