@@ -894,11 +894,13 @@ _pygi_argument_from_object (PyObject   *object,
 {
     GIArgument arg;
     GITypeTag type_tag;
+    gpointer cleanup_data = NULL;
 
     memset(&arg, 0, sizeof(GIArgument));
     type_tag = g_type_info_get_tag (type_info);
 
-    if (_pygi_marshal_from_py_basic_type (object, &arg, type_tag, transfer) ||
+    /* Ignores cleanup data for now. */
+    if (_pygi_marshal_from_py_basic_type (object, &arg, type_tag, transfer, &cleanup_data) ||
             PyErr_Occurred()) {
         return arg;
     }
@@ -1031,12 +1033,12 @@ array_success:
                                                             &arg,
                                                             NULL, /*arg_name*/
                                                             info, /*interface_info*/
-                                                            type_info,
                                                             g_type,
                                                             py_type,
                                                             transfer,
                                                             FALSE, /*copy_reference*/
-                                                            g_struct_info_is_foreign (info));
+                                                            g_struct_info_is_foreign (info),
+                                                            g_type_info_is_pointer (type_info));
 
                     Py_DECREF (py_type);
                     break;
