@@ -42,6 +42,7 @@ PyGIArgCache * _arg_cache_new_for_interface (GIInterfaceInfo *iface_info,
                                              PyGIDirection direction,
                                              gssize c_arg_index,
                                              gssize py_arg_index);
+
 /* cleanup */
 static void
 _pygi_arg_cache_free (PyGIArgCache *cache)
@@ -103,23 +104,17 @@ _callback_cache_free_func (PyGICallbackCache *cache)
 void
 _pygi_callable_cache_free (PyGICallableCache *cache)
 {
-    gssize i;
-
     if (cache == NULL)
         return;
 
     g_slist_free (cache->to_py_args);
     g_slist_free (cache->arg_name_list);
     g_hash_table_destroy (cache->arg_name_hash);
+    g_ptr_array_unref (cache->args_cache);
 
-    for (i = 0; i < cache->n_args; i++) {
-        PyGIArgCache *tmp = cache->args_cache[i];
-        _pygi_arg_cache_free (tmp);
-    }
     if (cache->return_cache != NULL)
         _pygi_arg_cache_free (cache->return_cache);
 
-    g_slice_free1 (cache->n_args * sizeof (PyGIArgCache *), cache->args_cache);
     g_slice_free (PyGICallableCache, cache);
 }
 
@@ -299,6 +294,78 @@ _arg_cache_to_py_utf8_setup (PyGIArgCache *arg_cache,
     arg_cache->to_py_cleanup = _pygi_marshal_cleanup_to_py_utf8;
 }
 
+static PyGIArgCache*
+_arg_cache_array_len_arg_setup (PyGIArgCache *arg_cache,
+                                PyGICallableCache *callable_cache,
+                                PyGIDirection direction,
+                                gssize arg_index,
+                                gssize *py_arg_index)
+{
+    PyGISequenceCache *seq_cache = (PyGISequenceCache *)arg_cache;
+    if (seq_cache->len_arg_index >= 0) {
+        PyGIArgCache *child_cache = NULL;
+
+        child_cache = _pygi_callable_cache_get_arg (callable_cache,
+                                                    seq_cache->len_arg_index);
+        if (child_cache == NULL) {
+            child_cache = _arg_cache_alloc ();
+        } else {
+            /* If the "length" arg cache already exists (the length comes before
+             * the array in the argument list), remove it from the to_py_args list
+             * because it does not belong in "to python" return tuple. The length
+             * will implicitly be a part of the returned Python list.
+             */
+            if (direction & PYGI_DIRECTION_TO_PYTHON) {
+                callable_cache->to_py_args =
+                    g_slist_remove (callable_cache->to_py_args, child_cache);
+            }
+
+            /* This is a case where the arg cache already exists and has been
+             * setup by another array argument sharing the same length argument.
+             * See: gi_marshalling_tests_multi_array_key_value_in
+             */
+            if (child_cache->meta_type == PYGI_META_ARG_TYPE_CHILD)
+                return child_cache;
+        }
+
+        /* There is a length argument for this array, so increment the number
+         * of "to python" child arguments when applicable.
+         */
+        if (direction & PYGI_DIRECTION_TO_PYTHON)
+             callable_cache->n_to_py_child_args++;
+
+        child_cache->meta_type = PYGI_META_ARG_TYPE_CHILD;
+        child_cache->direction = direction;
+        child_cache->to_py_marshaller = NULL;
+        child_cache->from_py_marshaller = NULL;
+
+        /* ugly edge case code:
+         *
+         * When the length comes before the array parameter we need to update
+         * indexes of arguments after the index argument.
+         */
+        if (seq_cache->len_arg_index < arg_index && direction & PYGI_DIRECTION_FROM_PYTHON) {
+            gssize i;
+            (*py_arg_index) -= 1;
+            callable_cache->n_py_args -= 1;
+
+            for (i = seq_cache->len_arg_index + 1;
+                   i < _pygi_callable_cache_args_len (callable_cache); i++) {
+                PyGIArgCache *update_cache = _pygi_callable_cache_get_arg (callable_cache, i);
+                if (update_cache == NULL)
+                    break;
+
+                update_cache->py_arg_index -= 1;
+            }
+        }
+
+        _pygi_callable_cache_set_arg (callable_cache, seq_cache->len_arg_index, child_cache);
+        return child_cache;
+    }
+
+    return NULL;
+}
+
 static gboolean
 _arg_cache_from_py_array_setup (PyGIArgCache *arg_cache,
                                 PyGICallableCache *callable_cache,
@@ -309,35 +376,8 @@ _arg_cache_from_py_array_setup (PyGIArgCache *arg_cache,
 {
     PyGISequenceCache *seq_cache = (PyGISequenceCache *)arg_cache;
     seq_cache->array_type = g_type_info_get_array_type (type_info);
-
     arg_cache->from_py_marshaller = _pygi_marshal_from_py_array;
-
-    if (seq_cache->len_arg_index >= 0) {
-        PyGIArgCache *child_cache = 
-            callable_cache->args_cache[seq_cache->len_arg_index];
-
-        if (child_cache == NULL) {
-            child_cache = _arg_cache_alloc ();
-        } else if (child_cache->meta_type == PYGI_META_ARG_TYPE_CHILD ||
-                   child_cache->meta_type == PYGI_META_ARG_TYPE_CHILD_NEEDS_UPDATE) {
-            arg_cache->from_py_cleanup = _pygi_marshal_cleanup_from_py_array;
-            return TRUE;
-        }
-
-        if (seq_cache->len_arg_index < arg_index)
-            child_cache->meta_type = PYGI_META_ARG_TYPE_CHILD_NEEDS_UPDATE;
-        else
-            child_cache->meta_type = PYGI_META_ARG_TYPE_CHILD;
-
-        child_cache->direction = direction;
-        child_cache->to_py_marshaller = NULL;
-        child_cache->from_py_marshaller = NULL;
-
-        callable_cache->args_cache[seq_cache->len_arg_index] = child_cache;
-    }
-
     arg_cache->from_py_cleanup = _pygi_marshal_cleanup_from_py_array;
-
     return TRUE;
 }
 
@@ -350,35 +390,9 @@ _arg_cache_to_py_array_setup (PyGIArgCache *arg_cache,
                               gssize arg_index)
 {
     PyGISequenceCache *seq_cache = (PyGISequenceCache *)arg_cache;
+    seq_cache->array_type = g_type_info_get_array_type (type_info);
     arg_cache->to_py_marshaller = _pygi_marshal_to_py_array;
     arg_cache->to_py_cleanup = _pygi_marshal_cleanup_to_py_array;
-
-    seq_cache->array_type = g_type_info_get_array_type (type_info);
-
-    if (seq_cache->len_arg_index >= 0) {
-        PyGIArgCache *child_cache = callable_cache->args_cache[seq_cache->len_arg_index];
-        if (seq_cache->len_arg_index < arg_index)
-             callable_cache->n_to_py_child_args++;
-
-        if (child_cache != NULL) {
-            callable_cache->to_py_args =
-                g_slist_remove (callable_cache->to_py_args, child_cache);
-
-            if (child_cache->meta_type == PYGI_META_ARG_TYPE_CHILD ||
-                child_cache->meta_type == PYGI_META_ARG_TYPE_CHILD_NEEDS_UPDATE)
-                return TRUE;
-        } else {
-            child_cache = _arg_cache_alloc ();
-        }
-
-        child_cache->meta_type = PYGI_META_ARG_TYPE_CHILD;
-        child_cache->direction = direction;
-        child_cache->to_py_marshaller = NULL;
-        child_cache->from_py_marshaller = NULL;
-
-        callable_cache->args_cache[seq_cache->len_arg_index] = child_cache;
-    }
-
     return TRUE;
 }
 
@@ -509,14 +523,17 @@ _arg_cache_from_py_interface_callback_setup (PyGIArgCache *arg_cache,
         PyGIArgCache *user_data_arg_cache = _arg_cache_alloc ();
         user_data_arg_cache->meta_type = PYGI_META_ARG_TYPE_CHILD_WITH_PYARG;
         user_data_arg_cache->direction = PYGI_DIRECTION_FROM_PYTHON;
-        callable_cache->args_cache[callback_cache->user_data_index] = user_data_arg_cache;
+        user_data_arg_cache->has_default = TRUE; /* always allow user data with a NULL default. */
+        _pygi_callable_cache_set_arg (callable_cache, callback_cache->user_data_index,
+                                      user_data_arg_cache);
     }
 
     if (callback_cache->destroy_notify_index >= 0) {
         PyGIArgCache *destroy_arg_cache = _arg_cache_alloc ();
         destroy_arg_cache->meta_type = PYGI_META_ARG_TYPE_CHILD;
         destroy_arg_cache->direction = PYGI_DIRECTION_FROM_PYTHON;
-        callable_cache->args_cache[callback_cache->destroy_notify_index] = destroy_arg_cache;
+        _pygi_callable_cache_set_arg (callable_cache, callback_cache->destroy_notify_index,
+                                      destroy_arg_cache);
     }
     arg_cache->from_py_marshaller = _pygi_marshal_from_py_interface_callback;
     arg_cache->from_py_cleanup = _pygi_marshal_cleanup_from_py_interface_callback;
@@ -589,31 +606,31 @@ _arg_cache_new_for_interface (GIInterfaceInfo *iface_info,
 
     switch (info_type) {
         case GI_INFO_TYPE_UNION:
-            if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_interface_union_setup (arg_cache, transfer);
 
-            if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_interface_union_setup (arg_cache, transfer);
 
             break;
         case GI_INFO_TYPE_BOXED:
         case GI_INFO_TYPE_STRUCT:
-            if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_interface_struct_setup (arg_cache,
                                                           iface_info,
                                                           transfer);
 
-            if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_interface_struct_setup (arg_cache,
                                                         iface_info,
                                                         transfer);
             break;
         case GI_INFO_TYPE_OBJECT:
         case GI_INFO_TYPE_INTERFACE:
-            if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_interface_object_setup (arg_cache, transfer);
 
-            if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_interface_object_setup (arg_cache, transfer);
 
             break;
@@ -621,7 +638,7 @@ _arg_cache_new_for_interface (GIInterfaceInfo *iface_info,
             {
                 PyGICallbackCache *callback_cache;
 
-                if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL) {
+                if (direction & PYGI_DIRECTION_TO_PYTHON) {
                     _arg_cache_to_py_interface_callback_setup ();
                     return NULL;
                 }
@@ -635,24 +652,24 @@ _arg_cache_new_for_interface (GIInterfaceInfo *iface_info,
                 if (arg_cache == NULL)
                     return NULL;
 
-                if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+                if (direction & PYGI_DIRECTION_FROM_PYTHON)
                     _arg_cache_from_py_interface_callback_setup (arg_cache, callable_cache);
 
                 break;
             }
         case GI_INFO_TYPE_ENUM:
-            if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_interface_enum_setup (arg_cache, transfer);
 
-            if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_interface_enum_setup (arg_cache, transfer);
 
             break;
         case GI_INFO_TYPE_FLAGS:
-            if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_interface_flags_setup (arg_cache, transfer);
 
-            if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+            if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_interface_flags_setup (arg_cache, transfer);
 
             break;
@@ -676,10 +693,30 @@ _arg_cache_new_for_interface (GIInterfaceInfo *iface_info,
     return arg_cache;
 }
 
+/* _arg_info_default_value
+ * info:
+ * arg: (out): GIArgument to fill in with default value.
+ *
+ * This is currently a place holder API which only supports "allow-none" pointer args.
+ * Once defaults are part of the GI API, we can replace this with: g_arg_info_default_value
+ * https://bugzilla.gnome.org/show_bug.cgi?id=558620
+ *
+ * Returns: TRUE if the given argument supports a default value and was filled in.
+ */
+static gboolean
+_arg_info_default_value (GIArgInfo *info, GIArgument *arg)
+{
+    if (g_arg_info_may_be_null (info)) {
+        arg->v_pointer = NULL;
+        return TRUE;
+    }
+    return FALSE;
+}
+
 PyGIArgCache *
 _arg_cache_new (GITypeInfo *type_info,
                 PyGICallableCache *callable_cache,
-                GIArgInfo *arg_info,
+                GIArgInfo *arg_info,     /* may be null */
                 GITransfer transfer,
                 PyGIDirection direction,
                 gssize c_arg_index,
@@ -702,10 +739,10 @@ _arg_cache_new (GITypeInfo *type_info,
            if (arg_cache == NULL)
                break;
 
-           if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_void_setup (arg_cache);
 
-           if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_void_setup (arg_cache);
 
            break;
@@ -726,10 +763,10 @@ _arg_cache_new (GITypeInfo *type_info,
            if (arg_cache == NULL)
                break;
 
-           if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_basic_type_setup (arg_cache);
 
-           if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_basic_type_setup (arg_cache);
 
            break;
@@ -739,10 +776,10 @@ _arg_cache_new (GITypeInfo *type_info,
            if (arg_cache == NULL)
                break;
 
-           if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_utf8_setup (arg_cache, transfer);
 
-           if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_utf8_setup (arg_cache, transfer);
 
            break;
@@ -758,7 +795,7 @@ _arg_cache_new (GITypeInfo *type_info,
                if (arg_cache == NULL)
                    break;
 
-               if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+               if (direction & PYGI_DIRECTION_FROM_PYTHON)
                    _arg_cache_from_py_array_setup (arg_cache,
                                                    callable_cache,
                                                    type_info,
@@ -766,7 +803,7 @@ _arg_cache_new (GITypeInfo *type_info,
                                                    direction,
                                                    c_arg_index);
 
-               if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+               if (direction & PYGI_DIRECTION_TO_PYTHON)
                    _arg_cache_to_py_array_setup (arg_cache,
                                                  callable_cache,
                                                  type_info,
@@ -774,31 +811,11 @@ _arg_cache_new (GITypeInfo *type_info,
                                                  direction,
                                                  c_arg_index);
 
-               /* ugly edge case code:
-                *  
-                * length can come before the array parameter which means we
-                * need to update indexes if this happens
-                */ 
-               if (seq_cache->len_arg_index > -1 &&
-                   callable_cache->args_cache[seq_cache->len_arg_index]->meta_type == PYGI_META_ARG_TYPE_CHILD_NEEDS_UPDATE) {
-                   gssize i;
-                   PyGIArgCache *child_cache =
-                       callable_cache->args_cache[seq_cache->len_arg_index];
-
-                   child_cache->meta_type = PYGI_META_ARG_TYPE_CHILD;
-                   py_arg_index -= 1;
-                   callable_cache->n_py_args -= 1;
-
-                   for (i = seq_cache->len_arg_index + 1; 
-                          i < callable_cache->n_args; 
-                            i++) {
-                       PyGIArgCache *update_cache = callable_cache->args_cache[i];
-                       if (update_cache == NULL)
-                           break;
-
-                       update_cache->py_arg_index -= 1;
-                   }
-               }
+               _arg_cache_array_len_arg_setup (arg_cache,
+                                               callable_cache,
+                                               direction,
+                                               c_arg_index,
+                                               &py_arg_index);
 
                break;
            }
@@ -814,10 +831,10 @@ _arg_cache_new (GITypeInfo *type_info,
                if (arg_cache == NULL)
                    break;
 
-               if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+               if (direction & PYGI_DIRECTION_FROM_PYTHON)
                    _arg_cache_from_py_glist_setup (arg_cache, transfer);
 
-               if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+               if (direction & PYGI_DIRECTION_TO_PYTHON)
                    _arg_cache_to_py_glist_setup (arg_cache, transfer);
 
 
@@ -835,10 +852,10 @@ _arg_cache_new (GITypeInfo *type_info,
                if (arg_cache == NULL)
                    break;
 
-               if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+               if (direction & PYGI_DIRECTION_FROM_PYTHON)
                    _arg_cache_from_py_gslist_setup (arg_cache, transfer);
 
-               if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+               if (direction & PYGI_DIRECTION_TO_PYTHON)
                    _arg_cache_to_py_gslist_setup (arg_cache, transfer);
 
                break;
@@ -852,10 +869,10 @@ _arg_cache_new (GITypeInfo *type_info,
            if (arg_cache == NULL)
                    break;
 
-           if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_ghash_setup (arg_cache);
 
-           if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL) {
+           if (direction & PYGI_DIRECTION_TO_PYTHON) {
                _arg_cache_to_py_ghash_setup (arg_cache);
            }
 
@@ -879,10 +896,10 @@ _arg_cache_new (GITypeInfo *type_info,
            if (arg_cache == NULL)
                break;
 
-           if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_FROM_PYTHON)
                _arg_cache_from_py_gerror_setup (arg_cache);
 
-           if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL)
+           if (direction & PYGI_DIRECTION_TO_PYTHON)
                _arg_cache_to_py_gerror_setup (arg_cache);
 
            break;
@@ -897,43 +914,41 @@ _arg_cache_new (GITypeInfo *type_info,
         arg_cache->is_pointer = g_type_info_is_pointer (type_info);
         g_base_info_ref ( (GIBaseInfo *) type_info);
         arg_cache->type_info = type_info;
+
+        if (arg_info != NULL) {
+            if (!arg_cache->has_default) {
+                /* It is possible has_default was set somewhere else */
+                arg_cache->has_default = _arg_info_default_value (arg_info,
+                                                                  &arg_cache->default_value);
+            }
+            arg_cache->arg_name = g_base_info_get_name ((GIBaseInfo *) arg_info);
+            arg_cache->allow_none = g_arg_info_may_be_null (arg_info);
+
+            if (type_tag == GI_TYPE_TAG_INTERFACE || type_tag == GI_TYPE_TAG_ARRAY)
+                arg_cache->is_caller_allocates = g_arg_info_is_caller_allocates (arg_info);
+            else
+                arg_cache->is_caller_allocates = FALSE;
+        }
     }
 
     return arg_cache;
 }
 
-static void
-_arg_name_list_generate (PyGICallableCache *callable_cache)
+static PyGIDirection
+_pygi_get_direction (PyGICallableCache *callable_cache, GIDirection gi_direction)
 {
-    GSList * arg_name_list = NULL;
-    int i;
-
-    if (callable_cache->arg_name_hash == NULL) {
-        callable_cache->arg_name_hash = g_hash_table_new (g_str_hash, g_str_equal);
+    /* For vfuncs and callbacks our marshalling directions are reversed */
+    if (gi_direction == GI_DIRECTION_INOUT) {
+        return PYGI_DIRECTION_BIDIRECTIONAL;
+    } else if (gi_direction == GI_DIRECTION_IN) {
+        if (callable_cache->function_type == PYGI_FUNCTION_TYPE_CALLBACK)
+            return PYGI_DIRECTION_TO_PYTHON;
+        return PYGI_DIRECTION_FROM_PYTHON;
     } else {
-        g_hash_table_remove_all (callable_cache->arg_name_hash);
+        if (callable_cache->function_type == PYGI_FUNCTION_TYPE_CALLBACK)
+            return PYGI_DIRECTION_FROM_PYTHON;
+        return PYGI_DIRECTION_TO_PYTHON;
     }
-
-    for (i=0; i < callable_cache->n_args; i++) {
-        PyGIArgCache *arg_cache = NULL;
-
-        arg_cache = callable_cache->args_cache[i];
-
-        if (arg_cache->meta_type != PYGI_META_ARG_TYPE_CHILD &&
-            arg_cache->meta_type != PYGI_META_ARG_TYPE_CLOSURE &&
-                (arg_cache->direction == PYGI_DIRECTION_FROM_PYTHON ||
-                 arg_cache->direction == PYGI_DIRECTION_BIDIRECTIONAL)) {
-
-            gpointer arg_name = (gpointer)arg_cache->arg_name;
-
-            arg_name_list = g_slist_prepend (arg_name_list, arg_name);
-            if (arg_name != NULL) {
-                g_hash_table_insert (callable_cache->arg_name_hash, arg_name, arg_name);
-            }
-        }
-    }
-
-    callable_cache->arg_name_list = g_slist_reverse (arg_name_list);
 }
 
 /* Generate the cache for the callable's arguments */
@@ -948,11 +963,8 @@ _args_cache_generate (GICallableInfo *callable_info,
     PyGIArgCache *return_cache;
     PyGIDirection return_direction;
 
-    /* determine if we are marshalling the return to or from python */
-    if (callable_cache->function_type == PYGI_FUNCTION_TYPE_CALLBACK)
-        return_direction = PYGI_DIRECTION_FROM_PYTHON;
-    else
-        return_direction = PYGI_DIRECTION_TO_PYTHON;
+    /* Return arguments are always considered out */
+    return_direction = _pygi_get_direction (callable_cache, GI_DIRECTION_OUT);
 
     /* cache the return arg */
     return_info =
@@ -979,10 +991,6 @@ _args_cache_generate (GICallableInfo *callable_info,
             callable_cache->function_type == PYGI_FUNCTION_TYPE_VFUNC) {
         GIInterfaceInfo *interface_info;
         PyGIArgCache *instance_cache;
-        PyGIDirection instance_direction;
-
-        instance_direction = PYGI_DIRECTION_FROM_PYTHON;
-
 
         interface_info = g_base_info_get_container ( (GIBaseInfo *)callable_info);
 
@@ -991,18 +999,20 @@ _args_cache_generate (GICallableInfo *callable_info,
                                           callable_cache,
                                           NULL,
                                           GI_TRANSFER_NOTHING,
-                                          instance_direction,
+                                          PYGI_DIRECTION_FROM_PYTHON,
                                           arg_index,
                                           0);
 
-        /* FIXME: marshal interfaces from_py */
-        instance_cache->from_py_marshaller = _pygi_marshal_from_py_interface_instance;
         g_base_info_unref ( (GIBaseInfo *)interface_info);
 
         if (instance_cache == NULL)
             return FALSE;
 
-        callable_cache->args_cache[arg_index] = instance_cache;
+        /* Because we are not supplied a GITypeInfo for instance arguments,
+         * assume some defaults. */
+        instance_cache->is_pointer = TRUE;
+
+        _pygi_callable_cache_set_arg (callable_cache, arg_index, instance_cache);
 
         arg_index++;
         callable_cache->n_from_py_args++;
@@ -1010,130 +1020,153 @@ _args_cache_generate (GICallableInfo *callable_info,
     }
 
 
-    for (i=0; arg_index < callable_cache->n_args; arg_index++, i++) {
+    for (i=0; arg_index < _pygi_callable_cache_args_len (callable_cache); arg_index++, i++) {
         PyGIArgCache *arg_cache = NULL;
         GIArgInfo *arg_info;
-        GITypeInfo *type_info;
-        GIDirection gi_direction;
         PyGIDirection direction;
-        GITransfer transfer;
-        GITypeTag type_tag;
-        gboolean is_caller_allocates = FALSE;
-        gssize py_arg_index = -1;
 
         arg_info = g_callable_info_get_arg (callable_info, i);
 
         if (g_arg_info_get_closure (arg_info) == i) {
 
             arg_cache = _arg_cache_alloc ();
-            callable_cache->args_cache[arg_index] = arg_cache;
+            _pygi_callable_cache_set_arg (callable_cache, arg_index, arg_cache);
 
-            arg_cache->arg_name = g_base_info_get_name ((GIBaseInfo *) arg_info);
-            arg_cache->direction = PYGI_DIRECTION_FROM_PYTHON;
+            direction = PYGI_DIRECTION_FROM_PYTHON;
+            arg_cache->direction = direction;
             arg_cache->meta_type = PYGI_META_ARG_TYPE_CLOSURE;
             arg_cache->c_arg_index = i;
 
             callable_cache->n_from_py_args++;
 
-            g_base_info_unref ( (GIBaseInfo *)arg_info);
-
-            continue;
-        }
-
-        /* For vfuncs and callbacks our marshalling directions
-           are reversed */
-        gi_direction = g_arg_info_get_direction (arg_info);
-        if (gi_direction == GI_DIRECTION_INOUT) {
-            direction = PYGI_DIRECTION_BIDIRECTIONAL;
-        } else if (gi_direction == GI_DIRECTION_IN) {
-            direction = PYGI_DIRECTION_FROM_PYTHON;
-            if (callable_cache->function_type == PYGI_FUNCTION_TYPE_CALLBACK)
-                direction = PYGI_DIRECTION_TO_PYTHON;
         } else {
-            direction = PYGI_DIRECTION_TO_PYTHON;
-            if (callable_cache->function_type == PYGI_FUNCTION_TYPE_CALLBACK)
-                direction = PYGI_DIRECTION_FROM_PYTHON;
-        }
+            GITypeInfo *type_info;
 
-        transfer = g_arg_info_get_ownership_transfer (arg_info);
-        type_info = g_arg_info_get_type (arg_info);
-        type_tag = g_type_info_get_tag (type_info);
+            direction = _pygi_get_direction (callable_cache,
+                                             g_arg_info_get_direction (arg_info));
+            type_info = g_arg_info_get_type (arg_info);
 
-        if (type_tag == GI_TYPE_TAG_INTERFACE || type_tag == GI_TYPE_TAG_ARRAY)
-            is_caller_allocates = g_arg_info_is_caller_allocates (arg_info);
+            /* must be an child arg filled in by its owner
+             * and continue
+             * fill in it's c_arg_index, add to the in count
+             */
+            arg_cache = _pygi_callable_cache_get_arg (callable_cache, arg_index);
+            if (arg_cache != NULL) {
+                if (arg_cache->meta_type == PYGI_META_ARG_TYPE_CHILD_WITH_PYARG) {
+                    arg_cache->py_arg_index = callable_cache->n_py_args;
+                    callable_cache->n_py_args++;
+                }
 
-        /* must be an child arg filled in by its owner
-         * and continue
-         * fill in it's c_arg_index, add to the in count
-         */
-        if (callable_cache->args_cache[arg_index] != NULL) {
-            arg_cache = callable_cache->args_cache[arg_index];
-            if (arg_cache->meta_type == PYGI_META_ARG_TYPE_CHILD_WITH_PYARG) {
-                arg_cache->py_arg_index = callable_cache->n_py_args;
-                callable_cache->n_py_args++;
+                if (direction & PYGI_DIRECTION_FROM_PYTHON) {
+                    arg_cache->c_arg_index = callable_cache->n_from_py_args;
+                    callable_cache->n_from_py_args++;
+                }
+
+                if (direction & PYGI_DIRECTION_TO_PYTHON) {
+                    callable_cache->n_to_py_args++;
+                }
+
+                arg_cache->type_tag = g_type_info_get_tag (type_info);
+
+            } else {
+                GITransfer transfer;
+                gssize py_arg_index = -1;
+                transfer = g_arg_info_get_ownership_transfer (arg_info);
+
+                if (direction & PYGI_DIRECTION_FROM_PYTHON) {
+                    py_arg_index = callable_cache->n_py_args;
+                    callable_cache->n_from_py_args++;
+                    callable_cache->n_py_args++;
+                }
+
+                arg_cache =
+                    _arg_cache_new (type_info,
+                                    callable_cache,
+                                    arg_info,
+                                    transfer,
+                                    direction,
+                                    arg_index,
+                                    py_arg_index);
+
+                if (arg_cache == NULL) {
+                    g_base_info_unref( (GIBaseInfo *)type_info);
+                    g_base_info_unref( (GIBaseInfo *)arg_info);
+                    return FALSE;
+                }
+
+
+                if (direction & PYGI_DIRECTION_TO_PYTHON) {
+                    callable_cache->n_to_py_args++;
+
+                    callable_cache->to_py_args =
+                        g_slist_append (callable_cache->to_py_args, arg_cache);
+                }
+
+                _pygi_callable_cache_set_arg (callable_cache, arg_index, arg_cache);
             }
 
-            if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL) {
-                arg_cache->c_arg_index = callable_cache->n_from_py_args;
-                callable_cache->n_from_py_args++;
-            }
-
-            if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL) {
-                callable_cache->n_to_py_args++;
-                callable_cache->n_to_py_child_args++;
-            }
-
-            arg_cache->type_tag = g_type_info_get_tag (type_info);
-
             g_base_info_unref (type_info);
-            g_base_info_unref ( (GIBaseInfo *)arg_info);
-            continue;
         }
 
-        if (direction == PYGI_DIRECTION_FROM_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL) {
-            py_arg_index = callable_cache->n_py_args;
-            callable_cache->n_from_py_args++;
-            callable_cache->n_py_args++;
-        }
-
-        arg_cache =
-            _arg_cache_new (type_info,
-                            callable_cache,
-                            arg_info,
-                            transfer,
-                            direction,
-                            arg_index,
-                            py_arg_index);
-
-        if (arg_cache == NULL)
-            goto arg_err;
-
+        /* Ensure arguments always have a name when available */
         arg_cache->arg_name = g_base_info_get_name ((GIBaseInfo *) arg_info);
-        arg_cache->allow_none = g_arg_info_may_be_null(arg_info);
-        arg_cache->is_caller_allocates = is_caller_allocates;
 
-        if (direction == PYGI_DIRECTION_TO_PYTHON || direction == PYGI_DIRECTION_BIDIRECTIONAL) {
-            callable_cache->n_to_py_args++;
+        g_base_info_unref ( (GIBaseInfo *)arg_info);
 
-            if (arg_cache == NULL)
-                goto arg_err;
-
-            callable_cache->to_py_args =
-                g_slist_append (callable_cache->to_py_args, arg_cache);
-        }
-
-        callable_cache->args_cache[arg_index] = arg_cache;
-        g_base_info_unref( (GIBaseInfo *)type_info);
-        g_base_info_unref( (GIBaseInfo *)arg_info);
-
-        continue;
-arg_err:
-        g_base_info_unref( (GIBaseInfo *)type_info);
-        g_base_info_unref( (GIBaseInfo *)arg_info);
-        return FALSE;
     }
 
-    _arg_name_list_generate (callable_cache);
+    if (callable_cache->arg_name_hash == NULL) {
+        callable_cache->arg_name_hash = g_hash_table_new (g_str_hash, g_str_equal);
+    } else {
+        g_hash_table_remove_all (callable_cache->arg_name_hash);
+    }
+    callable_cache->n_py_required_args = 0;
+    callable_cache->user_data_varargs_index = -1;
+
+    gssize last_explicit_arg_index = -1;
+
+    /* Reverse loop through all the arguments to setup arg_name_list/hash
+     * and find the number of required arguments */
+    for (i=((gssize)_pygi_callable_cache_args_len (callable_cache))-1; i >= 0; i--) {
+        PyGIArgCache *arg_cache = _pygi_callable_cache_get_arg (callable_cache, i);
+
+        if (arg_cache->meta_type != PYGI_META_ARG_TYPE_CHILD &&
+                arg_cache->meta_type != PYGI_META_ARG_TYPE_CLOSURE &&
+                arg_cache->direction & PYGI_DIRECTION_FROM_PYTHON) {
+
+            /* Setup arg_name_list and arg_name_hash */
+            gpointer arg_name = (gpointer)arg_cache->arg_name;
+            callable_cache->arg_name_list = g_slist_prepend (callable_cache->arg_name_list,
+                                                             arg_name);
+            if (arg_name != NULL) {
+                g_hash_table_insert (callable_cache->arg_name_hash,
+                                     arg_name,
+                                     GINT_TO_POINTER(i));
+            }
+
+            /* The first tail argument without a default will force all the preceding
+             * argument defaults off. This limits support of default args to the
+             * tail of an args list.
+             */
+            if (callable_cache->n_py_required_args > 0) {
+                arg_cache->has_default = FALSE;
+                callable_cache->n_py_required_args += 1;
+            } else if (!arg_cache->has_default) {
+                callable_cache->n_py_required_args += 1;
+            }
+
+            if (last_explicit_arg_index == -1) {
+                last_explicit_arg_index = i;
+
+                /* If the last "from python" argument in the args list is a child
+                 * with pyarg (currently only callback user_data). Set it to eat
+                 * variable args in the callable cache.
+                 */
+                if (arg_cache->meta_type == PYGI_META_ARG_TYPE_CHILD_WITH_PYARG)
+                    callable_cache->user_data_varargs_index = i;
+            }
+        }
+    }
 
     return TRUE;
 }
@@ -1141,6 +1174,7 @@ arg_err:
 PyGICallableCache *
 _pygi_callable_cache_new (GICallableInfo *callable_info, gboolean is_ccallback)
 {
+    gint n_args;
     PyGICallableCache *cache;
     GIInfoType type = g_base_info_get_type ( (GIBaseInfo *)callable_info);
 
@@ -1185,15 +1219,17 @@ _pygi_callable_cache_new (GICallableInfo *callable_info, gboolean is_ccallback)
         cache->function_type = PYGI_FUNCTION_TYPE_METHOD;
     }
 
-    cache->n_args = g_callable_info_get_n_args (callable_info);
+    n_args = g_callable_info_get_n_args (callable_info);
 
     /* if we are a method or vfunc make sure the instance parameter is counted */
     if (cache->function_type == PYGI_FUNCTION_TYPE_METHOD ||
             cache->function_type == PYGI_FUNCTION_TYPE_VFUNC)
-        cache->n_args++;
+        n_args++;
 
-    if (cache->n_args > 0)
-        cache->args_cache = g_slice_alloc0 (cache->n_args * sizeof (PyGIArgCache *));
+    if (n_args >= 0) {
+        cache->args_cache = g_ptr_array_new_full (n_args, (GDestroyNotify) _pygi_arg_cache_free);
+        g_ptr_array_set_size (cache->args_cache, n_args);
+    }
 
     if (!_args_cache_generate (callable_info, cache))
         goto err;
