@@ -120,7 +120,11 @@ _check_for_unexpected_kwargs (const gchar *function_name,
             }
         }
 
-        if (g_hash_table_lookup (arg_name_hash, PyBytes_AsString(key)) == NULL) {
+        /* Use extended lookup because it returns whether or not the key actually
+         * exists in the hash table. g_hash_table_lookup returns NULL for keys not
+         * found which maps to index 0 for our hash lookup.
+         */
+        if (!g_hash_table_lookup_extended (arg_name_hash, PyBytes_AsString(key), NULL, NULL)) {
             PyErr_Format (PyExc_TypeError,
                           "%.200s() got an unexpected keyword argument '%.400s'",
                           function_name,
@@ -161,12 +165,12 @@ _py_args_combine_and_check_length (PyGICallableCache *cache,
 
     /* Fast path, we already have the exact number of args and not kwargs. */
     n_expected_args = g_slist_length (cache->arg_name_list);
-    if (n_py_kwargs == 0 && n_py_args == n_expected_args) {
+    if (n_py_kwargs == 0 && n_py_args == n_expected_args && cache->user_data_varargs_index < 0) {
         Py_INCREF (py_args);
         return py_args;
     }
 
-    if (n_expected_args < n_py_args) {
+    if (cache->user_data_varargs_index < 0 && n_expected_args < n_py_args) {
         PyErr_Format (PyExc_TypeError,
                       "%.200s() takes exactly %d %sargument%s (%zd given)",
                       function_name,
@@ -177,6 +181,13 @@ _py_args_combine_and_check_length (PyGICallableCache *cache,
         return NULL;
     }
 
+    if (cache->user_data_varargs_index >= 0 && n_py_kwargs > 0 && n_expected_args < n_py_args) {
+        PyErr_Format (PyExc_TypeError,
+                      "%.200s() cannot use variable user data arguments with keyword arguments",
+                      function_name);
+        return NULL;
+    }
+
     if (n_py_kwargs > 0 && !_check_for_unexpected_kwargs (function_name,
                                                           cache->arg_name_hash,
                                                           py_kwargs)) {
@@ -187,39 +198,73 @@ _py_args_combine_and_check_length (PyGICallableCache *cache,
      * when they are combined into a single tuple */
     combined_py_args = PyTuple_New (n_expected_args);
 
-    for (i = 0; i < n_py_args; i++) {
-        PyObject *item = PyTuple_GET_ITEM (py_args, i);
-        Py_INCREF (item);
-        PyTuple_SET_ITEM (combined_py_args, i, item);
-    }
-
     for (i = 0, l = cache->arg_name_list; i < n_expected_args && l; i++, l = l->next) {
-        PyObject *py_arg_item, *kw_arg_item = NULL;
+        PyObject *py_arg_item = NULL;
+        PyObject *kw_arg_item = NULL;
         const gchar *arg_name = l->data;
+        int arg_cache_index = -1;
+        gboolean is_varargs_user_data = FALSE;
+
+        if (arg_name != NULL)
+            arg_cache_index = GPOINTER_TO_INT (g_hash_table_lookup (cache->arg_name_hash, arg_name));
+
+        is_varargs_user_data = cache->user_data_varargs_index >= 0 &&
+                                arg_cache_index == cache->user_data_varargs_index;
 
         if (n_py_kwargs > 0 && arg_name != NULL) {
             /* NULL means this argument has no keyword name */
             /* ex. the first argument to a method or constructor */
             kw_arg_item = PyDict_GetItemString (py_kwargs, arg_name);
         }
-        py_arg_item = PyTuple_GET_ITEM (combined_py_args, i);
 
-        if (kw_arg_item != NULL && py_arg_item == NULL) {
-            Py_INCREF (kw_arg_item);
-            PyTuple_SET_ITEM (combined_py_args, i, kw_arg_item);
+        /* use a bounded retrieval of the original input */
+        if (i < n_py_args)
+            py_arg_item = PyTuple_GET_ITEM (py_args, i);
+
+        if (kw_arg_item == NULL && py_arg_item != NULL) {
+            if (is_varargs_user_data) {
+                /* For tail end user_data varargs, pull a slice off and we are done. */
+                PyObject *user_data = PyTuple_GetSlice (py_args, i, PY_SSIZE_T_MAX);
+                PyTuple_SET_ITEM (combined_py_args, i, user_data);
+                return combined_py_args;
+            } else {
+                Py_INCREF (py_arg_item);
+                PyTuple_SET_ITEM (combined_py_args, i, py_arg_item);
+            }
+        } else if (kw_arg_item != NULL && py_arg_item == NULL) {
+            if (is_varargs_user_data) {
+                /* Special case where user_data is passed as a keyword argument (user_data=foo)
+                 * Wrap the value in a tuple to represent variable args for marshaling later on.
+                 */
+                PyObject *user_data = Py_BuildValue("(O)", kw_arg_item, NULL);
+                PyTuple_SET_ITEM (combined_py_args, i, user_data);
+            } else {
+                Py_INCREF (kw_arg_item);
+                PyTuple_SET_ITEM (combined_py_args, i, kw_arg_item);
+            }
 
         } else if (kw_arg_item == NULL && py_arg_item == NULL) {
-            PyErr_Format (PyExc_TypeError,
-                          "%.200s() takes exactly %d %sargument%s (%zd given)",
-                          function_name,
-                          n_expected_args,
-                          n_py_kwargs > 0 ? "non-keyword " : "",
-                          n_expected_args == 1 ? "" : "s",
-                          n_py_args);
-
-            Py_DECREF (combined_py_args);
-            return NULL;
+            if (is_varargs_user_data) {
+                /* For varargs user_data, pass an empty tuple when nothing is given. */
+                PyTuple_SET_ITEM (combined_py_args, i, PyTuple_New (0));
+            } else if (arg_cache_index >= 0 && _pygi_callable_cache_get_arg (cache, arg_cache_index)->has_default) {
+                /* If the argument supports a default, use a place holder in the
+                 * argument tuple, this will be checked later during marshaling.
+                 */
+                Py_INCREF (_PyGIDefaultArgPlaceholder);
+                PyTuple_SET_ITEM (combined_py_args, i, _PyGIDefaultArgPlaceholder);
+            } else {
+                PyErr_Format (PyExc_TypeError,
+                              "%.200s() takes exactly %d %sargument%s (%zd given)",
+                              function_name,
+                              n_expected_args,
+                              n_py_kwargs > 0 ? "non-keyword " : "",
+                              n_expected_args == 1 ? "" : "s",
+                              n_py_args);
 
+                Py_DECREF (combined_py_args);
+                return NULL;
+            }
         } else if (kw_arg_item != NULL && py_arg_item != NULL) {
             PyErr_Format (PyExc_TypeError,
                           "%.200s() got multiple values for keyword argument '%.200s'",
@@ -299,14 +344,14 @@ _invoke_state_init_from_callable_cache (PyGIInvokeState *state,
     }
     state->n_py_in_args = PyTuple_Size (state->py_in_args);
 
-    state->args = g_slice_alloc0 (cache->n_args * sizeof (GIArgument *));
-    if (state->args == NULL && cache->n_args != 0) {
+    state->args = g_slice_alloc0 (_pygi_callable_cache_args_len (cache) * sizeof (GIArgument *));
+    if (state->args == NULL && _pygi_callable_cache_args_len (cache) != 0) {
         PyErr_NoMemory();
         return FALSE;
     }
 
-    state->args_data = g_slice_alloc0 (cache->n_args * sizeof (gpointer));
-    if (state->args_data == NULL && cache->n_args != 0) {
+    state->args_cleanup_data = g_slice_alloc0 (_pygi_callable_cache_args_len (cache) * sizeof (gpointer));
+    if (state->args_cleanup_data == NULL && _pygi_callable_cache_args_len (cache) != 0) {
         PyErr_NoMemory();
         return FALSE;
     }
@@ -337,8 +382,8 @@ _invoke_state_init_from_callable_cache (PyGIInvokeState *state,
 static inline void
 _invoke_state_clear (PyGIInvokeState *state, PyGICallableCache *cache)
 {
-    g_slice_free1 (cache->n_args * sizeof(GIArgument *), state->args);
-    g_slice_free1 (cache->n_args * sizeof(gpointer), state->args_data);
+    g_slice_free1 (_pygi_callable_cache_args_len (cache) * sizeof(GIArgument *), state->args);
+    g_slice_free1 (_pygi_callable_cache_args_len (cache) * sizeof(gpointer), state->args_cleanup_data);
     g_slice_free1 (cache->n_from_py_args * sizeof(GIArgument), state->in_args);
     g_slice_free1 (cache->n_to_py_args * sizeof(GIArgument), state->out_args);
     g_slice_free1 (cache->n_to_py_args * sizeof(GIArgument), state->out_values);
@@ -408,9 +453,9 @@ _invoke_marshal_in_args (PyGIInvokeState *state, PyGICallableCache *cache)
         return FALSE;
     }
 
-    for (i = 0; i < cache->n_args; i++) {
+    for (i = 0; i < _pygi_callable_cache_args_len (cache); i++) {
         GIArgument *c_arg;
-        PyGIArgCache *arg_cache = cache->args_cache[i];
+        PyGIArgCache *arg_cache = g_ptr_array_index (cache->args_cache, i);
         PyObject *py_arg = NULL;
 
         switch (arg_cache->direction) {
@@ -490,8 +535,12 @@ _invoke_marshal_in_args (PyGIInvokeState *state, PyGICallableCache *cache)
         }
 
         c_arg = state->args[i];
-        if (arg_cache->from_py_marshaller != NULL) {
+        if (py_arg == _PyGIDefaultArgPlaceholder) {
+            *c_arg = arg_cache->default_value;
+        } else if (arg_cache->from_py_marshaller != NULL) {
             gboolean success;
+            gpointer cleanup_data = NULL;
+
             if (!arg_cache->allow_none && py_arg == Py_None) {
                 PyErr_Format (PyExc_TypeError,
                               "Argument %zd does not allow None as a value",
@@ -503,10 +552,13 @@ _invoke_marshal_in_args (PyGIInvokeState *state, PyGICallableCache *cache)
                  return FALSE;
             }
             success = arg_cache->from_py_marshaller (state,
-                                                              cache,
-                                                              arg_cache,
-                                                              py_arg,
-                                                              c_arg);
+                                                     cache,
+                                                     arg_cache,
+                                                     py_arg,
+                                                     c_arg,
+                                                     &cleanup_data);
+            state->args_cleanup_data[i] = cleanup_data;
+
             if (!success) {
                 pygi_marshal_cleanup_args_from_py_parameter_fail (state,
                                                                   cache,
@@ -563,6 +615,7 @@ _invoke_marshal_out_args (PyGIInvokeState *state, PyGICallableCache *cache)
                 if (to_py_cleanup != NULL)
                     to_py_cleanup ( state,
                                     cache->return_cache,
+                                    NULL,
                                    &state->return_arg,
                                     FALSE);
             }
@@ -652,9 +705,9 @@ pygi_callable_info_invoke (GIBaseInfo *info, PyObject *py_args,
     if (!_invoke_callable (&state, cache, info, function_ptr))
         goto err;
 
+    ret = _invoke_marshal_out_args (&state, cache);
     pygi_marshal_cleanup_args_from_py_marshal_success (&state, cache);
 
-    ret = _invoke_marshal_out_args (&state, cache);
     if (ret)
         pygi_marshal_cleanup_args_to_py_marshal_success (&state, cache);
 err:
