@@ -314,12 +314,12 @@ class ExceptHook(object):
     are never bubbled through from python to C back to python.
     This works because exception hooks are called in PyErr_Print.
     """
-    def __init__(self, exc_type):
-        self._exc_type = exc_type
+    def __init__(self, *expected_exc_types):
+        self._expected_exc_types = expected_exc_types
         self._exceptions = []
 
     def _excepthook(self, exc_type, value, traceback):
-        self._exceptions.append(exc_type)
+        self._exceptions.append((exc_type, value))
 
     def __enter__(self):
         self._oldhook = sys.excepthook
@@ -328,8 +328,13 @@ class ExceptHook(object):
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         sys.excepthook = self._oldhook
-        assert len(self._exceptions) == 1, 'Expecting exactly one exception of type %s' % self._exc_type
-        assert issubclass(self._exceptions[0], self._exc_type), 'Expecting exactly one exception of type %s' % self._exc_type
+        error_message = 'Expecting the following exceptions: %s, got: %s' % \
+            (str(self._expected_exc_types), '\n'.join([str(item) for item in self._exceptions]))
+
+        assert len(self._expected_exc_types) == len(self._exceptions), error_message
+
+        for expected, got in zip(self._expected_exc_types, [exc[0] for exc in self._exceptions]):
+            assert issubclass(got, expected), error_message
 
 
 class TestReturnsAfterError(unittest.TestCase):
@@ -347,7 +352,7 @@ class TestReturnsAfterError(unittest.TestCase):
         self.assertEqual(count, 0)
 
     def test_get_column_type(self):
-        with ExceptHook(NotImplementedError):
+        with ExceptHook(NotImplementedError, TypeError):
             col_type = self.model.get_column_type(0)
         self.assertEqual(col_type, GObject.TYPE_INVALID)
 
