@@ -6,6 +6,7 @@ import contextlib
 import unittest
 import time
 import sys
+import warnings
 
 from compathelper import _unicode, _bytes
 
@@ -16,8 +17,10 @@ from gi.repository import GLib, GObject
 try:
     from gi.repository import GdkPixbuf, Gdk, Gtk
     Gtk  # pyflakes
+    PyGTKDeprecationWarning = Gtk.PyGTKDeprecationWarning
 except ImportError:
     Gtk = None
+    PyGTKDeprecationWarning = None
 
 
 @contextlib.contextmanager
@@ -73,16 +76,14 @@ class TestGtk(unittest.TestCase):
 
     def test_actions(self):
         self.assertEqual(Gtk.Action, gi.overrides.Gtk.Action)
-        self.assertRaises(TypeError, Gtk.Action)
-        action = Gtk.Action("test", "Test", "Test Action", Gtk.STOCK_COPY)
+        action = Gtk.Action(name="test", label="Test", tooltip="Test Action", stock_id=Gtk.STOCK_COPY)
         self.assertEqual(action.get_name(), "test")
         self.assertEqual(action.get_label(), "Test")
         self.assertEqual(action.get_tooltip(), "Test Action")
         self.assertEqual(action.get_stock_id(), Gtk.STOCK_COPY)
 
         self.assertEqual(Gtk.RadioAction, gi.overrides.Gtk.RadioAction)
-        self.assertRaises(TypeError, Gtk.RadioAction)
-        action = Gtk.RadioAction("test", "Test", "Test Action", Gtk.STOCK_COPY, 1)
+        action = Gtk.RadioAction(name="test", label="Test", tooltip="Test Action", stock_id=Gtk.STOCK_COPY, value=1)
         self.assertEqual(action.get_name(), "test")
         self.assertEqual(action.get_label(), "Test")
         self.assertEqual(action.get_tooltip(), "Test Action")
@@ -91,7 +92,6 @@ class TestGtk(unittest.TestCase):
 
     def test_actiongroup(self):
         self.assertEqual(Gtk.ActionGroup, gi.overrides.Gtk.ActionGroup)
-        self.assertRaises(TypeError, Gtk.ActionGroup)
 
         action_group = Gtk.ActionGroup(name='TestActionGroup')
         callback_data = "callback data"
@@ -165,10 +165,6 @@ class TestGtk(unittest.TestCase):
         w = Gtk.Window(type=Gtk.WindowType.POPUP)
         self.assertEqual(w.get_property('type'), Gtk.WindowType.POPUP)
 
-        # pygtk compatible positional argument
-        w = Gtk.Window(Gtk.WindowType.POPUP)
-        self.assertEqual(w.get_property('type'), Gtk.WindowType.POPUP)
-
         class TestWindow(Gtk.Window):
             __gtype_name__ = "TestWindow"
 
@@ -192,26 +188,80 @@ class TestGtk(unittest.TestCase):
         self.assertEqual(builder.get_object('testpop').get_property('type'),
                          Gtk.WindowType.POPUP)
 
-    def test_dialogs(self):
+    def test_dialog_classes(self):
         self.assertEqual(Gtk.Dialog, gi.overrides.Gtk.Dialog)
-        self.assertEqual(Gtk.AboutDialog, gi.overrides.Gtk.AboutDialog)
-        self.assertEqual(Gtk.MessageDialog, gi.overrides.Gtk.MessageDialog)
         self.assertEqual(Gtk.ColorSelectionDialog, gi.overrides.Gtk.ColorSelectionDialog)
         self.assertEqual(Gtk.FileChooserDialog, gi.overrides.Gtk.FileChooserDialog)
         self.assertEqual(Gtk.FontSelectionDialog, gi.overrides.Gtk.FontSelectionDialog)
         self.assertEqual(Gtk.RecentChooserDialog, gi.overrides.Gtk.RecentChooserDialog)
 
-        # Gtk.Dialog
-        dialog = Gtk.Dialog(title='Foo',
-                            flags=Gtk.DialogFlags.MODAL,
-                            buttons=('test-button1', 1))
+    def test_dialog_base(self):
+        dialog = Gtk.Dialog(title='Foo', modal=True)
         self.assertTrue(isinstance(dialog, Gtk.Dialog))
         self.assertTrue(isinstance(dialog, Gtk.Window))
-
-        dialog.add_buttons('test-button2', 2, Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
-
         self.assertEqual('Foo', dialog.get_title())
         self.assertTrue(dialog.get_modal())
+
+    def test_dialog_deprecations(self):
+        with warnings.catch_warnings(record=True) as warn:
+            warnings.simplefilter('always')
+            dialog = Gtk.Dialog(title='Foo', flags=Gtk.DialogFlags.MODAL)
+            self.assertTrue(dialog.get_modal())
+            self.assertEqual(len(warn), 1)
+            self.assertTrue(issubclass(warn[0].category, PyGTKDeprecationWarning))
+            self.assertRegexpMatches(str(warn[0].message),
+                                     '.*flags.*modal.*')
+
+        with warnings.catch_warnings(record=True) as warn:
+            warnings.simplefilter('always')
+            dialog = Gtk.Dialog(title='Foo', flags=Gtk.DialogFlags.DESTROY_WITH_PARENT)
+            self.assertTrue(dialog.get_destroy_with_parent())
+            self.assertEqual(len(warn), 1)
+            self.assertTrue(issubclass(warn[0].category, PyGTKDeprecationWarning))
+            self.assertRegexpMatches(str(warn[0].message),
+                                     '.*flags.*destroy_with_parent.*')
+
+    def test_dialog_deprecation_stacklevels(self):
+        # Test warning levels are setup to give the correct filename for
+        # deprecations in different classes in the inheritance hierarchy.
+
+        # Base class
+        self.assertEqual(Gtk.Dialog, gi.overrides.Gtk.Dialog)
+        with warnings.catch_warnings(record=True) as warn:
+            warnings.simplefilter('always')
+            Gtk.Dialog(flags=Gtk.DialogFlags.MODAL)
+            self.assertEqual(len(warn), 1)
+            self.assertRegexpMatches(warn[0].filename, '.*test_overrides_gtk.*')
+
+        # Validate overridden base with overridden sub-class.
+        self.assertEqual(Gtk.MessageDialog, gi.overrides.Gtk.MessageDialog)
+        with warnings.catch_warnings(record=True) as warn:
+            warnings.simplefilter('always')
+            Gtk.MessageDialog(flags=Gtk.DialogFlags.MODAL)
+            self.assertEqual(len(warn), 1)
+            self.assertRegexpMatches(warn[0].filename, '.*test_overrides_gtk.*')
+
+        # Validate overridden base with non-overridden sub-class.
+        self.assertEqual(Gtk.AboutDialog, gi.repository.Gtk.AboutDialog)
+        with warnings.catch_warnings(record=True) as warn:
+            warnings.simplefilter('always')
+            Gtk.AboutDialog(flags=Gtk.DialogFlags.MODAL)
+            self.assertEqual(len(warn), 1)
+            self.assertRegexpMatches(warn[0].filename, '.*test_overrides_gtk.*')
+
+    def test_dialog_add_buttons(self):
+        # The overloaded "buttons" keyword gives a warning when attempting
+        # to use it for adding buttons as was available in PyGTK.
+        with warnings.catch_warnings(record=True) as warn:
+            warnings.simplefilter('always')
+            dialog = Gtk.Dialog(title='Foo', modal=True,
+                                buttons=('test-button1', 1))
+            self.assertEqual(len(warn), 1)
+            self.assertTrue(issubclass(warn[0].category, PyGTKDeprecationWarning))
+            self.assertRegexpMatches(str(warn[0].message),
+                                     '.*ButtonsType.*add_buttons.*')
+
+        dialog.add_buttons('test-button2', 2, Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
         button = dialog.get_widget_for_response(1)
         self.assertEqual('test-button1', button.get_label())
         button = dialog.get_widget_for_response(2)
@@ -219,16 +269,20 @@ class TestGtk(unittest.TestCase):
         button = dialog.get_widget_for_response(Gtk.ResponseType.CLOSE)
         self.assertEqual(Gtk.STOCK_CLOSE, button.get_label())
 
-        # Gtk.AboutDialog
+    def test_about_dialog(self):
         dialog = Gtk.AboutDialog()
         self.assertTrue(isinstance(dialog, Gtk.Dialog))
         self.assertTrue(isinstance(dialog, Gtk.Window))
 
-        # Gtk.MessageDialog
+        # AboutDialog is not sub-classed in overrides, make sure
+        # the mro still injects the base class "add_buttons" override.
+        self.assertTrue(hasattr(dialog, 'add_buttons'))
+
+    def test_message_dialog(self):
         dialog = Gtk.MessageDialog(title='message dialog test',
-                                   flags=Gtk.DialogFlags.MODAL,
+                                   modal=True,
                                    buttons=Gtk.ButtonsType.OK,
-                                   message_format='dude!')
+                                   text='dude!')
         self.assertTrue(isinstance(dialog, Gtk.Dialog))
         self.assertTrue(isinstance(dialog, Gtk.Window))
 
@@ -245,58 +299,53 @@ class TestGtk(unittest.TestCase):
         self.assertEqual(dialog.get_property('secondary-text'), '2nd markup')
         self.assertTrue(dialog.get_property('secondary-use-markup'))
 
-        # Gtk.ColorSelectionDialog
-        dialog = Gtk.ColorSelectionDialog("color selection dialog test")
+    def test_color_selection_dialog(self):
+        dialog = Gtk.ColorSelectionDialog(title="color selection dialog test")
         self.assertTrue(isinstance(dialog, Gtk.Dialog))
         self.assertTrue(isinstance(dialog, Gtk.Window))
         self.assertEqual('color selection dialog test', dialog.get_title())
 
-        # Gtk.FileChooserDialog
+    def test_file_chooser_dialog(self):
         # might cause a GVFS warning, do not break on this
         old_mask = GLib.log_set_always_fatal(
             GLib.LogLevelFlags.LEVEL_CRITICAL | GLib.LogLevelFlags.LEVEL_ERROR)
         try:
             dialog = Gtk.FileChooserDialog(title='file chooser dialog test',
-                                           buttons=('test-button1', 1),
                                            action=Gtk.FileChooserAction.SAVE)
         finally:
             GLib.log_set_always_fatal(old_mask)
         self.assertTrue(isinstance(dialog, Gtk.Dialog))
         self.assertTrue(isinstance(dialog, Gtk.Window))
-
-        dialog.add_buttons('test-button2', 2, Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
         self.assertEqual('file chooser dialog test', dialog.get_title())
-        button = dialog.get_widget_for_response(1)
-        self.assertEqual('test-button1', button.get_label())
-        button = dialog.get_widget_for_response(2)
-        self.assertEqual('test-button2', button.get_label())
-        button = dialog.get_widget_for_response(Gtk.ResponseType.CLOSE)
-        self.assertEqual(Gtk.STOCK_CLOSE, button.get_label())
+
         action = dialog.get_property('action')
         self.assertEqual(Gtk.FileChooserAction.SAVE, action)
 
-        # Gtk.FontSelectionDialog
-        dialog = Gtk.ColorSelectionDialog("font selection dialog test")
+    def test_file_chooser_dialog_default_action(self):
+        # might cause a GVFS warning, do not break on this
+        old_mask = GLib.log_set_always_fatal(
+            GLib.LogLevelFlags.LEVEL_CRITICAL | GLib.LogLevelFlags.LEVEL_ERROR)
+        try:
+            dialog = Gtk.FileChooserDialog(title='file chooser dialog test')
+        finally:
+            GLib.log_set_always_fatal(old_mask)
+
+        action = dialog.get_property('action')
+        self.assertEqual(Gtk.FileChooserAction.OPEN, action)
+
+    def test_font_selection_dialog(self):
+        dialog = Gtk.FontSelectionDialog(title="font selection dialog test")
         self.assertTrue(isinstance(dialog, Gtk.Dialog))
         self.assertTrue(isinstance(dialog, Gtk.Window))
         self.assertEqual('font selection dialog test', dialog.get_title())
 
-        # Gtk.RecentChooserDialog
+    def test_recent_chooser_dialog(self):
         test_manager = Gtk.RecentManager()
         dialog = Gtk.RecentChooserDialog(title='recent chooser dialog test',
-                                         buttons=('test-button1', 1),
-                                         manager=test_manager)
+                                         recent_manager=test_manager)
         self.assertTrue(isinstance(dialog, Gtk.Dialog))
         self.assertTrue(isinstance(dialog, Gtk.Window))
-
-        dialog.add_buttons('test-button2', 2, Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
         self.assertEqual('recent chooser dialog test', dialog.get_title())
-        button = dialog.get_widget_for_response(1)
-        self.assertEqual('test-button1', button.get_label())
-        button = dialog.get_widget_for_response(2)
-        self.assertEqual('test-button2', button.get_label())
-        button = dialog.get_widget_for_response(Gtk.ResponseType.CLOSE)
-        self.assertEqual(Gtk.STOCK_CLOSE, button.get_label())
 
     class TestClass(GObject.GObject):
         __gtype_name__ = "GIOverrideTreeAPITest"
@@ -319,7 +368,7 @@ class TestGtk(unittest.TestCase):
         self.assertTrue(isinstance(button, Gtk.Button))
         self.assertTrue(isinstance(button, Gtk.Container))
         self.assertTrue(isinstance(button, Gtk.Widget))
-        button = Gtk.Button(stock=Gtk.STOCK_CLOSE)
+        button = Gtk.Button.new_from_stock(Gtk.STOCK_CLOSE)
         self.assertEqual(Gtk.STOCK_CLOSE, button.get_label())
         self.assertTrue(button.get_use_stock())
         self.assertTrue(button.get_use_underline())
@@ -331,8 +380,7 @@ class TestGtk(unittest.TestCase):
         self.assertTrue(button.get_use_underline())
 
         # test Gtk.LinkButton
-        self.assertRaises(TypeError, Gtk.LinkButton)
-        button = Gtk.LinkButton('http://www.Gtk.org', 'Gtk')
+        button = Gtk.LinkButton(uri='http://www.Gtk.org', label='Gtk')
         self.assertTrue(isinstance(button, Gtk.Button))
         self.assertTrue(isinstance(button, Gtk.Container))
         self.assertTrue(isinstance(button, Gtk.Widget))
@@ -392,25 +440,21 @@ class TestGtk(unittest.TestCase):
         self.assertEqual(adjustment.get_page_size(), page_size)
 
     def test_adjustment(self):
-        adjustment = Gtk.Adjustment(1, 0, 6, 4, 5, 3)
-        self.adjustment_check(adjustment, 1, 0, 6, 4, 5, 3)
+        adjustment = Gtk.Adjustment(value=1, lower=0, upper=6, step_increment=4, page_increment=5, page_size=3)
+        self.adjustment_check(adjustment, value=1, lower=0, upper=6, step_increment=4, page_increment=5, page_size=3)
 
-        adjustment = Gtk.Adjustment(1, 0, 6, 4, 5)
-        self.adjustment_check(adjustment, 1, 0, 6, 4, 5)
+        adjustment = Gtk.Adjustment(value=1, lower=0, upper=6, step_increment=4, page_increment=5)
+        self.adjustment_check(adjustment, value=1, lower=0, upper=6, step_increment=4, page_increment=5)
 
-        adjustment = Gtk.Adjustment(1, 0, 6, 4)
-        self.adjustment_check(adjustment, 1, 0, 6, 4)
+        adjustment = Gtk.Adjustment(value=1, lower=0, upper=6, step_increment=4)
+        self.adjustment_check(adjustment, value=1, lower=0, upper=6, step_increment=4)
 
-        adjustment = Gtk.Adjustment(1, 0, 6)
-        self.adjustment_check(adjustment, 1, 0, 6)
+        adjustment = Gtk.Adjustment(value=1, lower=0, upper=6)
+        self.adjustment_check(adjustment, value=1, lower=0, upper=6)
 
         adjustment = Gtk.Adjustment()
         self.adjustment_check(adjustment)
 
-        adjustment = Gtk.Adjustment(value=1, lower=0, upper=6,
-                                    step_increment=4, page_increment=5, page_size=3)
-        self.adjustment_check(adjustment, 1, 0, 6, 4, 5, 3)
-
     def test_table(self):
         table = Gtk.Table()
         self.assertTrue(isinstance(table, Gtk.Table))
@@ -418,17 +462,11 @@ class TestGtk(unittest.TestCase):
         self.assertTrue(isinstance(table, Gtk.Widget))
         self.assertEqual(table.get_size(), (1, 1))
         self.assertEqual(table.get_homogeneous(), False)
-        table = Gtk.Table(2, 3)
+
+        table = Gtk.Table(n_rows=2, n_columns=3)
         self.assertEqual(table.get_size(), (2, 3))
         self.assertEqual(table.get_homogeneous(), False)
-        table = Gtk.Table(2, 3, True)
-        self.assertEqual(table.get_size(), (2, 3))
-        self.assertEqual(table.get_homogeneous(), True)
 
-        # Test PyGTK interface
-        table = Gtk.Table(rows=3, columns=2)
-        self.assertEqual(table.get_size(), (3, 2))
-        # Test using the actual property names
         table = Gtk.Table(n_rows=2, n_columns=3, homogeneous=True)
         self.assertEqual(table.get_size(), (2, 3))
         self.assertEqual(table.get_homogeneous(), True)
@@ -522,7 +560,6 @@ class TestGtk(unittest.TestCase):
                                         Gdk.DragAction.DEFAULT | Gdk.DragAction.MOVE)
 
     def test_scrollbar(self):
-        # PyGTK compat
         adjustment = Gtk.Adjustment()
 
         hscrollbar = Gtk.HScrollbar()
@@ -530,8 +567,8 @@ class TestGtk(unittest.TestCase):
         self.assertNotEqual(hscrollbar.props.adjustment, adjustment)
         self.assertNotEqual(vscrollbar.props.adjustment, adjustment)
 
-        hscrollbar = Gtk.HScrollbar(adjustment)
-        vscrollbar = Gtk.VScrollbar(adjustment)
+        hscrollbar = Gtk.HScrollbar(adjustment=adjustment)
+        vscrollbar = Gtk.VScrollbar(adjustment=adjustment)
         self.assertEqual(hscrollbar.props.adjustment, adjustment)
         self.assertEqual(vscrollbar.props.adjustment, adjustment)
 
@@ -541,7 +578,7 @@ class TestGtk(unittest.TestCase):
         self.assertEqual(iconview.props.model, None)
 
         model = Gtk.ListStore(str)
-        iconview = Gtk.IconView(model)
+        iconview = Gtk.IconView(model=model)
         self.assertEqual(iconview.props.model, model)
 
     def test_toolbutton(self):
@@ -549,7 +586,7 @@ class TestGtk(unittest.TestCase):
         button = Gtk.ToolButton()
         self.assertEqual(button.props.stock_id, None)
 
-        button = Gtk.ToolButton('gtk-new')
+        button = Gtk.ToolButton(stock_id='gtk-new')
         self.assertEqual(button.props.stock_id, 'gtk-new')
 
         icon = Gtk.Image.new_from_stock(Gtk.STOCK_OPEN, Gtk.IconSize.SMALL_TOOLBAR)
@@ -559,13 +596,11 @@ class TestGtk(unittest.TestCase):
         self.assertEqual(button.props.icon_widget, icon)
 
     def test_iconset(self):
-        # PyGTK compat
         Gtk.IconSet()
         pixbuf = GdkPixbuf.Pixbuf()
-        Gtk.IconSet(pixbuf)
+        Gtk.IconSet.new_from_pixbuf(pixbuf)
 
     def test_viewport(self):
-        # PyGTK compat
         vadjustment = Gtk.Adjustment()
         hadjustment = Gtk.Adjustment()
 
@@ -758,9 +793,8 @@ class TestBuilder(unittest.TestCase):
 class TestTreeModel(unittest.TestCase):
     def test_tree_model_sort(self):
         self.assertEqual(Gtk.TreeModelSort, gi.overrides.Gtk.TreeModelSort)
-        self.assertRaises(TypeError, Gtk.TreeModelSort)
         model = Gtk.TreeStore(int, bool)
-        model_sort = Gtk.TreeModelSort(model)
+        model_sort = Gtk.TreeModelSort(model=model)
         self.assertEqual(model_sort.get_model(), model)
 
     def test_tree_store(self):
@@ -1565,10 +1599,13 @@ class TestTreeView(unittest.TestCase):
 
     def test_tree_view_column(self):
         cell = Gtk.CellRendererText()
-        Gtk.TreeViewColumn(title='This is just a test',
-                           cell_renderer=cell,
-                           text=0,
-                           style=2)
+        col = Gtk.TreeViewColumn(title='This is just a test',
+                                 cell_renderer=cell,
+                                 text=0,
+                                 style=2)
+
+        # Regression test for: https://bugzilla.gnome.org/show_bug.cgi?id=711173
+        col.set_cell_data_func(cell, None, None)
 
     def test_tree_view_add_column_with_attributes(self):
         model = Gtk.ListStore(str, str, str)
@@ -1577,7 +1614,7 @@ class TestTreeView(unittest.TestCase):
         model.append(['cell13', 'cell11', 'cell12'])
         model.append(['cell23', 'cell21', 'cell22'])
 
-        tree = Gtk.TreeView(model)
+        tree = Gtk.TreeView(model=model)
         cell1 = Gtk.CellRendererText()
         cell2 = Gtk.CellRendererText()
         cell3 = Gtk.CellRendererText()
